package com.dobsondev;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

/**
 * This class is used with the A* program. This class creates Map
 * objects that represent the maps that A* will path find through.
 * Each map is a grid of Nodes, which is a separate class.
 * 
 * @author Alex Dobson
 *
 */
public class Map {

	private String filename;
	private Node[][] nodes;
	
	private int width, height;
	
	/**
	 * The constructor for the Map takes an input file which represents the map
	 * with character's, and then creates the Nodes accordingly. This constructor
	 * only checks if the file is a real file or not, and assuming it is a file
	 * calls create map to parse the file and create the Nodes.
	 * 
	 * @param filename - the map file
	 * @throws FileNotFoundException
	 */
	public Map(String filename) throws FileNotFoundException {
		this.filename = filename;
		try {
			createMap();
		} catch (FileNotFoundException e) { throw new FileNotFoundException(); }
	}
	
	/**
	 * This method parses the input file given to the constructor to create Nodes
	 * for the Map object. The parsing works by first reading in the width and
	 * height of the map which are integers at the top of the file. Following this
	 * the method reads each character in the row and if it is a 'M' then that Node
	 * is traversable, and if it is a '#' then that Node is not traversable.
	 * 
	 * @throws FileNotFoundException
	 */
	private void createMap() throws FileNotFoundException {
		File map_file = new File(filename);
		try {
			Scanner in = new Scanner(map_file);
			
			width = in.nextInt();
			height = in.nextInt();
			
			nodes = new Node[width][height];
			
			int y = 0;
			while (in.hasNext()) {
				String row = in.next();
				
				for (int x = 0; x < row.length(); x++) {
					if (row.charAt(x) == 'M') {
						nodes[x][y] = new Node(x, y, true);
					} else {
						nodes[x][y] = new Node(x, y, false);
					}
				}
				y++;
			}
		} catch (FileNotFoundException e) { throw new FileNotFoundException(); }
	}//end createMovableGrid
	
	/********* GETTERS *********/
	public Node getNode(int x, int y) {
		return nodes[x][y];
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	public String getFileName() {
		return filename;
	}
	
	/**
	 * This method is used to print the map out onto the command line during the
	 * course of running Main. A Node can be printed as a '-' if it is traversable,
	 * a '#' if it is not traversable, and a '*' if it is traversable and is part
	 * of the optimal path.
	 */
	public void print() {
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				if (nodes[x][y].isTraversable() && !nodes[x][y].isInPath()) {
					System.out.print(" - ");
				} else if (nodes[x][y].isTraversable() && nodes[x][y].isInPath()) {
					System.out.print(" * ");
				} else {
					System.out.print(" # ");
				}
			}
			System.out.println("");
		}
	}//end print
	
}//end public class Map
