package com.dobsondev;

/**
 * This class is used to represent a node on a grid based map for use in the Main.java
 * class of this project. Each node has a x-coordinate, y-coordinate, along with three
 * different values that help the A* algorithm run. The node also has two boolean
 * values that distinguish if the node is traversable or if its in the optimal path.
 * Each node also has a parent (if the parent is set) in order to help A* build
 * the optimal path once its found it.
 * 
 * @author Alex Dobson
 *         University of Alberta, Augustana Campus
 *         2013-04-01
 *
 */
public class Node {

	private int x, y; 
	private double h_value, g_value, f_value;
	private boolean traversable, in_path;
	private Node parent;
	
	/**
	 * The constructor for Node takes in a x-coordinate, y-coordinate and if the Node
	 * is traversable or not and simply sets these values accordingly. By default each
	 * Node is also not in the path (to the goal) so this value is set to false.
	 * 
	 * @param x - the x-coordinate of the node
	 * @param y - the y-coordinate of the node
	 * @param traversable - if the node can be traversed
	 */
	public Node(int x, int y, boolean traversable) {
		this.x = x;
		this.y = y;
		this.traversable = traversable;
		in_path = false;
	}
	
	/********* GETTERS *********/
	public int getX() {
		return x;
	}
	
	public int getY() {
		return y;
	}
	
	public double getH_value() {
		return h_value;
	}
	
	public double getG_value() {
		return g_value;	
	}
	
	public double getF_value() {
		return f_value;
	}
	
	public boolean isTraversable() {
		return traversable;
	}
	
	public boolean isInPath() {
		return in_path;
	}
	
	public Node getParent() {
		return parent;
	}
	
	/********* SETTERS *********/
	public void setH_value(double h_value) {
		this.h_value = h_value;
	}
	
	public void setG_value(double g_value) {
		this.g_value = g_value;
	}
	
	public void setF_value() {
		f_value = g_value + h_value;
	}
	
	public void setInPath(boolean in_path) {
		this.in_path = in_path;
	}
	
	public void setParent(Node parent) {
		this.parent = parent;
	}
	
	/**
	 * The print method is used to print out important information about each node to
	 * the command line should the user need it. It is used with the testing mode
	 * of Main.
	 */
	public void print() {
		System.out.println("Node (" + x + ", " + y + ")");
		System.out.println("     h-value: " + h_value);
		System.out.println("     g-value: " + g_value);
		System.out.println("     f-value: " + f_value);
		if (parent != null) {
			System.out.println("     parent: Node (" + parent.getX() + ", " + parent.getY() + ")");
		} else {
			System.out.println("     parent: NULL");
		}
		System.out.println("     in path: " + in_path);
	}//end print
	
}//end public class Node
